import sys
sys.path.append('CutHelper')
from scipy import signal
from typing import Any
import math
import cv2
from CameraCut import CameraCutParam , CameraOuterParam
import numpy as np
from enum import unique , Enum , auto
from CutHelper.FlowHelper import FlowHelper

PI = 3.1415926
@unique
class CutType(Enum) : 
    DisparityCut = auto() 
    DepthCut = auto() 
    FlowCut = auto() 

class GradType(Enum):
    LineGrad = auto()
    ConvGrad = auto()
    AngleGrad = auto()
    PlaneGrad = auto()


class CutHelper:

    def __init__(self , cols, rows, leftCamera: CameraCutParam , rightCamera: CameraCutParam , outParam : CameraOuterParam , cutType :int) :
       
        self.m_leftParam = leftCamera 
        self.m_rightParam = rightCamera
        self.m_outerParam = outParam
        # self.m_rightParam = CameraCutParam(3277.61,3277.13 , 554.28 , 365.937)
        # self.m_leftParam = CameraCutParam(3252.27 ,3245.92 , 601.061 ,388.031)
        # R = np.array([[0.997795 , -0.00468741 , -0.0662074],
        #               [0.00360465 , 0.999858 , -0.016464],
        #               [0.0662752,0.0161891,0.99767]])
        # t = np.array([127.54,5.79367 ,389.958 ]).reshape(1,3)
        # self.m_outerParam = CameraOuterParam(R , t)

        if cutType == CutType.FlowCut.value : 
            self.m_cutType = CutType.FlowCut
            self.m_cutter = FlowHelper(cols , rows)
            self.m_cutter.initCameraParam(self.m_leftParam , self.m_rightParam , self.m_outerParam) 

        elif cutType == CutType.DisparityCut.value : 
            #TODO
            self.m_cutType = CutType.DisparityCut
        else :
            #TODO
            self.m_cutType = CutType.DepthCut

        
        # flowObj = flowHelper(flow_up[0].shape[1],flow_up[0].shape[0] )
        
    def SetJumpParam(self , x , y):
        self.m_jumpPixelX = x
        self.m_jumpPixelY = y
        return 
    
    def GetJumpParam(self) : 
        return self.m_jumpPixelX , self.m_jumpPixelY
    
    def SetPixelFilter(self ,minSize , maxSize) :
        self.m_iminSize = minSize 
        self.m_imaxSize = maxSize 
        return 
    def GetPixelFilter(self) :
        return self.m_iminSize , self.m_imaxSize
    
    def SetAngleDivide(self , angle_x , angle_y) :
        self.m_angleDivide_x = angle_x
        self.m_angleDivide_y = angle_y
        return 
    def GetAngleDivide(self):
        return self.m_angleDivide_x , self.m_angleDivide_y
    
    def SegPic(self, srcImg: cv2.Mat ,  gradType: int, roiInfo= None) :
        if self.m_cutType == CutType.DisparityCut : 
            numPos , cutBlock , tick , multiple = self.cutWithDisparity(srcImg , gradType , roiInfo)
        elif self.m_cutType == CutType.DepthCut : 
            return
            numPos , cutBlock , tick , multiple = self.cutWithDepth(srcImg , gradType , roiInfo)
            #TODO
        else: #self.m_cutType == CutType.FlowCut :
            depthpic ,cutplus , dividePic , tick = self.cutWithFlow(srcImg , gradType , roiInfo)
            retSlices , depthpic  = self.backProccesed(depthpic , dividePic , tick)
            return  retSlices , depthpic , cutplus
        pass
        # return self.backProccesed(cutBlock , srcImg.shape[0] , srcImg.shape[1] , tick , multiple , numPos)
        
    def cutWithDepth(self , flow_up , gradType:int , roiInfo = None ) :
        #TODO
        return 
    
    def cutWithFlow(self, flow_up:list , gradType: int , roiInfo  = None) :
        if gradType == GradType.PlaneGrad.value:
            return self.m_cutter.PlaneCut(flow_up , roiInfo)
        elif gradType == GradType.ConvGrad.value :
            depthpic ,cutplus = self.m_cutter.Flow2Depth( flow_up , roiInfo)
            # exit()
            fusionDevide , xDevide , yDevide = self.convGradDepthImg(cutplus)
            dividePic , tick = self.fusionDepthCut(fusionDevide,xDevide,yDevide)
            # ret = self.convGradFlow(flow_up[0],flow_up[1])
            
            return depthpic , cutplus , dividePic , tick
            # return self.flowPlaneGrad()

        pass        
        

    def cutWithDisparity(self, srcImg:cv2.Mat , gradType: int , roiInfo  = None) :

        kxCitaMap = srcImg
        kyCitaMap =  srcImg

        if gradType == GradType.LineGrad.value:
            self.LineGrad( kxCitaMap , kyCitaMap,roiInfo) 
            
        tmpCita = 0.0
        rows = kxCitaMap.shape[0]
        cols = kxCitaMap.shape[1]
        for i in range(0 , rows - 1) :
            for j in range(0 , cols - 1) :
                if j == cols -1 :
                    kxCitaMap[j][i] = PI
                    continue
                tmpCita = abs(kxCitaMap[j][i] - kxCitaMap[j+1][i])
                tmpCita = PI - tmpCita
                kxCitaMap[j][i] = tmpCita

        for i in range(0 , rows-1) :
            for j in range(0 , cols -1 ) :
                if i== rows - 1 :
                    kyCitaMap[j][i] = PI
                    continue
                tmpCita = abs(kyCitaMap[j][i] - kyCitaMap[j][i+1])
                tmpCita = PI - tmpCita
                kyCitaMap[j][i] = tmpCita
        
        #对于夹角值的展示
        # minCita , maxCita = cv2.minMaxLoc(kxCitaMap)
        # kxCutShow = kxCitaMap * 255.0 / (maxCita - minCita) - minCita / (maxCita - minCita)
        # cv2.imshow("kxCut" , kxCutShow)

        # minCita , maxCita = cv2.minMaxLoc(kyCitaMap)
        # kyCutShow = kyCitaMap * 255.0 / (maxCita - minCita) - minCita / (maxCita - minCita)
        # cv2.imshow("kyCut" , kyCutShow)
        # cv2.waitKey(0)

        return self.devicdeCut(kxCitaMap , kyCitaMap)
    def backProccesed(self , depthpic:np.array , dividePic:np.array , tick:int ): 
        # print("start backProccesed")

        retSlices = []
        # expandConv=np.array([[1,1,1],[1,1,1],[1,1,1]])
        # kernel = np.ones((3,3), np.uint8)
        #在这里建立一个数组，若融合过程中与其他的类融在一起，则后续直接continue

        numPos1 = [0] * (tick+1)
        numPos1[0] = 1
        for i in range(1 , tick+1) :
            if numPos1[i] >0 :

                continue

            tmpSlice = np.where(dividePic == i , 1.0 , 0.0)
            # cuttmpSlice = np.where(dividePic == 0  , 0. , -1.)
            # tmpSlice = tmpSlice + cuttmpSlice # -1区域：其他颜色，1:区域：自己选中的信息 ，0：边界
            # tmpSlice = cv2.morphologyEx(tmpSlice, cv2.MORPH_CLOSE, kernel)
            # tmpSlice = signal.convolve2d(tmpSlice , expandConv , 'same') 
            # tmpSlice = signal.convolve2d(tmpSlice , expandConv , 'same') 
            # tmpSlice = signal.convolve2d(tmpSlice , expandConv , 'same')  #之前边界扩张了3次，现在内部也往外扩3次
            tmpSlice = np.where(tmpSlice > 0 , 255 , 0) #在这里固化当前的区域
            #在这里融合
            tmpDivide =  tmpSlice *dividePic 
            for j in range(i+1 , tick+1):
                if (np.where(tmpDivide==j*255,1,0)).sum() >0:
                    numPos1[j] = 1

                    tmpAdd = np.where(dividePic == j, 255 , 0)
                    tmpSlice = tmpAdd+tmpSlice

            tmpSlice = np.where(tmpSlice > 0 , 255 , 0) #在这里固化当前的区域
            # print("pos[" , i ,"]:" ,tmpSlice.sum()/255)

            #与最大最小pixel比对进行过滤
            if tmpSlice.sum() > 255 * self.m_iminSize and tmpSlice.sum() < 255*self.m_imaxSize :
                retSlices += [tmpSlice]
            #     print(i , tmpSlice.sum()/255 )
                # cv2.imwrite('demo_output/PixelArea'+ str(i) + '.jpg' , tmpSlice)

        return retSlices , depthpic
    
    # def backProccesed(self , cutSlices: np.array , rows:int , cols:int , tick: int ,multiple: float , numPos: []): 
    #     print("start backProccesed")
    #     pixels = 0 
    #     retSlices = []
    #     for k in range(0 , tick) :
    #         pixels = 0
    #         tmpSlice = np.zeros([rows, cols] , dtype=int)
    #         for i in range(0 ,cols) :
    #             for j in range(0 , rows) :
    #                 if numPos[cutSlices[i][j]] == k+1 :
    #                     tmpSlice[i][j] = 255
    #                     pixels+=1
    #                 else:
    #                     tmpSlice[i][j] = 0
        
    #         if self.m_cutParam.m_iminSize > 0 and self.m_cutParam.m_imaxSize :
    #             if pixels >= self.m_cutParam.m_iminSize and pixels <= self.m_cutParam.m_imaxSize :
    #                 retSlices.append(tmpSlice)
        
    #     # for i in range(0 , retSlices.count-1) :
    #     #     #存储单张黑白图片以供调试
    #     #     imgName = "./retSlices" + str(i) + ".jpg"
    #     #     cv2.imwrite(imgName , retSlices[i])

    #     Slices = cv2.Mat([rows, cols], dtype=cv2.CV_8UC1)

    #     for i in range(0 , rows-1):
    #         for j in range(0 , cols -1) :
    #             Slices[i][j] = int(multiple * numPos[cutSlices[i][j]])

    #     return retSlices , Slices

    def devicdeCut(self , kxCitaMap: cv2.Mat , kyCitaMap: cv2.Mat ) :
        colorType = 1 
        diffPixels = 0
        divide = PI / 180 * self.m_cutParam.m_angleDivide 

        rows = kxCitaMap.shape[1]
        cols = kxCitaMap.shape[0]

        cutBlock = np.zeros([rows, cols])

        for i in range(0 , rows - 1 ):
            for j in range(0 , cols - 1) :
                if kxCitaMap[j][i] < divide :
                    if diffPixels > self.m_cutParam.m_jumpPixelX :
                        j = j - diffPixels
                        cutBlock[j][i] = colorType
                        colorType+=1
                        diffPixels = 0
                    else:
                        diffPixels+=1 
                else:
                    if diffPixels > 0 :
                        for q in range(j - diffPixels , j-1) :
                            cutBlock[q][i] = colorType
                        diffPixels = 0
                    cutBlock[j][i] = colorType
            if diffPixels > 0 :
                colorType += 1
                for q in range(cols - diffPixels , cols -1):
                    cutBlock[q][i] = colorType
                colorType += 1

        print( "color type after x-divide : %d" , colorType )

        numPos = [i in range(0, colorType-1)]
        
        #列融合
        minor = 0                
        for j in range(0 , cols -1 ) :
            for i in range(0 , rows - 2) :
                if cutBlock[j][i+1] != cutBlock[j][i] :
                    if numPos[cutBlock[j][i]] != numPos[cutBlock[j][i+1]] :
                        if kyCitaMap[j][i] > divide :
                            minor = min( numPos[cutBlock[j][i]] , numPos[cutBlock[j][i+1]] )
                            numPos[cutBlock[j][i]] = minor
                            numPos[cutBlock[j][i+1]]  = minor

        tick = 1 
        isUsed = False

        for k  in range(1 , colorType):
            isUsed = False
            for i in range(1 , colorType) :
                if numPos[i] == k :
                    isUsed = True
                    numPos[i] = tick
            
            if isUsed:
                tick+= 1
        
        #计算重复色彩数

        multiple = 255.0 / tick
        print("after y-divide , tangle(180):" ,self.m_cutParam.m_angleDivide , ",tick num : " ,  tick , ",multiple : " , multiple )

        return numPos , cutBlock , tick , multiple

    def LineGrad(self , kxMap: cv2.Mat  , kyMap : cv2.Mat , roiInfo = None) :
        kxMap = self.dispX2Cita(kxMap,roiInfo)
        kyMap = self.dispY2Cita(kyMap,roiInfo)
        return
    
    def npdispX2Cita(self , srcImg: cv2.Mat , roiInfo = None) :
        #TODO:对视差分割的性能优化
        #bias:矩形框相对的位置
        if roiInfo :
            bias= roiInfo[0] 
        else:
            bias = 0.
        rows = srcImg.shape [0] # height
        cols = srcImg.shape [1] # width
        
        padding = np.zeros(rows,dtype=float)
        leftImg = np.insert(srcImg,0,padding,axis=1)
        rightImg = np.insert(srcImg,cols,padding,axis=1)
        xPos = np.arange(cols+1) + np.ones_like(bias-self.m_cutParam.m_cx)
        print(xPos)
        print(rightImg- leftImg)
        exit(0)
        # tmpCita = np.arctan2((rightImg - leftImg) * (np.arange(cols+1) + bias - self.m_cutParam.m_cx) + rightImg )
        # print(tmpCita)
        # return tmpCita
     
    def dispX2Cita(self , srcImg: cv2.Mat , roiInfo = None) : 
        #bias:矩形框相对的位置
        if roiInfo :
            bias= roiInfo[0] 
        else:
            bias = 0 
        tmpCita = 0.0
        rows = srcImg.shape [0] # height
        cols = srcImg.shape [1] # width
        for i in range( 0 , cols -1 ) :
            for j in range(0 , rows - 2) :
                tmpCita = srcImg[j][i]*(i + bias + 1 - self.m_cutParam.m_cx) - srcImg[j+1][i] * (i + bias - self.m_cutParam.m_cx)
                if tmpCita == 0.0 :
                    srcImg[j][i] = PI / 2
                else:
                    tmpCita = self.m_cutParam.m_fx *(srcImg[j+1][i] - srcImg[j][i]) / tmpCita
                    tmpCita = math.atan(tmpCita)
                    srcImg[j][i] = tmpCita
        #补一下最右边一列的值
        for i in range(0 , rows-1): 
            srcImg[i][cols-1] = srcImg[i][cols -2]
    
    def dispY2Cita(self , srcImg: cv2.Mat , roiInfo = None) :
        if roiInfo :
            bias= roiInfo[1]
        else:
            bias = 0 
        tmpCita = 0.0

        rows = srcImg.shape [0] # height
        cols = srcImg.shape [1] # width

        for i in range(0 , cols  -2 ):
            for j in range(0 , rows-1) :
                tmpCita = srcImg[j][i] * (j +bias + 1 -self.m_cutParam.m_cy) - srcImg[j][i+1]*(j + bias - self.m_cutParam.m_cy)
                if tmpCita == 0 :
                    srcImg[j][i] = PI/2
                else :
                    tmpCita = self.m_cutParam.m_fy *(srcImg[j][i+1] - srcImg[j][i] )*(i + bias - self.m_cutParam.m_cy)
                    tmpCita = math.atan(tmpCita)
                    srcImg[j][i] = tmpCita
        for i in range(0 , cols -1) :
            srcImg[rows-1][i] = srcImg[rows-2][i]
            

    def convGradDepthImg(self,depthImg: np.array) :
        rows = depthImg.shape[0]
        cols = depthImg.shape[1]
        # coreConv = np.array([[1,-2,1],[2,-4,2,],[1,-2,1]]) / 4
        coreConvX = np.array([[-1,1]])
        kernel = np.ones((5,5), np.uint8)
        # coreConvX = np.array([[-3,0,3],[-10,0,10],[-3,0,3]])/16
        retX = signal.convolve2d(depthImg , coreConvX , 'same') 
        retX = np.arctan(retX) #X方向上的角度,除以2是因为跨度为2
        retX = signal.convolve2d(retX , coreConvX , 'same')  #x方向上的角度差
        retX = abs(retX)  #直接取绝对值，若大于该值则为连续的，否则为分离的
        Xmask = np.where(retX<(self.m_angleDivide_x*np.pi / 180 ) , 0. , 1. )
        for i in range(0 , rows) :
            click = 0
            for j in range(0 , cols) :
                if Xmask[i][j] > 0  :
                    click += 1
                else:
                    if click <= self.m_jumpPixelX :
                        for d in range(click) :
                            Xmask[i][j-d-1] = 0
                    click = 0
        # Xmask = retX * Xmask
        Xmask = cv2.morphologyEx(Xmask, cv2.MORPH_CLOSE, kernel)
        Xmask = np.where(Xmask > 0 , 1 , 0)
        retX = retX * Xmask
        
        Xdeivde = np.where(Xmask > 0 , 255 , 0)
        # retXshow = ((retX - retX.min()) * 255 / (retX.max() - retX.min()) ).astype(np.uint8)
        # print(retXshow)
        # retXshow = cv2.applyColorMap(retXshow, 2)
        cv2.imwrite("demo_output/retX.jpg" , Xdeivde) 


        coreConvY = np.array([[-1],[1]])
        # coreConvY = np.array([[-3,-10,-3],[0,0,0],[3,10,3]])/16
        retY = signal.convolve2d(depthImg , coreConvY , 'same')
        retY = np.arctan(retY ) #Y方向上的角度
        retY = signal.convolve2d(retY , coreConvY , 'same') #y方向上的角度差
        retY = abs(retY)
        Ymask = np.where(retY<(self.m_angleDivide_y*np.pi / 180 ) , 0. , 1. )
        for j in range(0 , cols) :
            click = 0
            for i in range(0 , rows) :
                if Ymask[i][j] > 0  :
                    click += 1
                else:
                    if click <= self.m_jumpPixelY :
                        for d in range(click) :
                            Ymask[i-d-1][j] = 0
                    click = 0
        # Ymask = retY * Ymask
        Ymask = cv2.morphologyEx(Ymask, cv2.MORPH_CLOSE, kernel)
        Ymask = np.where(Ymask > 0 , 1 , 0)
        retY = retY * Ymask
        
        Ydeivde = np.where(Ymask > 0 , 255 , 0)

        # print(retY.max() , retY.min())
        # retYshow = ((retY-retY.min()) * 255 / (retY.max() - retY.min()) ).astype(np.uint8)
        # print(retYshow)
        # retYshow = cv2.applyColorMap(retYshow, 2) 
        
 
        
        cv2.imwrite("demo_output/retY.jpg" , Ydeivde) 
       

        fusionXY = retX + retY
        fusionXYShow = np.where(fusionXY > 0 , 255 , 0)
        # fusionXYShow = ((fusionXY - fusionXY.min())*255. / (fusionXY.max() - fusionXY.min()) ).astype(np.uint8)
        # print(fusionXY)
        # fusionXYShow = fusionXY
        # fusionXYShow = cv2.applyColorMap(fusionXYShow, 2) 
        
        # cv2.imwrite("demo_output/fusionXY.jpg" , fusionXYShow) 
        return fusionXY , retX , retY
    
    def convGradFlow(self, flow0:np.array ,flow1:np.array) :
        #这里是光流卷积的可视化,暂时没用
        coreConvX = np.array([[0,0,0],[-1 , 0 ,1 ],[0,0,0]])
        retX = signal.convolve2d(flow0 , coreConvX , 'same')
        retX = signal.convolve2d(retX , coreConvX , 'same')
        print(retX)
        # retXshow = np.where(retX > 0 , 255 , 0 ) 
        retXshow = ((retX - retX.min()) * 255 / (retX.max() - retX.min()) ).astype(np.uint8)
        # print(retXshow)
        retXshow = cv2.applyColorMap(retXshow, 2)
        # cv2.imwrite("demo_output/retX.jpg" , retXshow) 

        coreConvY = np.array([[0,-1,0],[0, 0 ,0],[0,1,0]])
        retY = signal.convolve2d(flow1 , coreConvY , 'same')
        retY = signal.convolve2d(retY , coreConvY , 'same')
        print(retY)
        # retYshow = np.where(retY > 0 , 255 , 0)
        retYshow = ((retY-retY.min()) * 255 / (retY.max() - retY.min()) ).astype(np.uint8)
        retYshow = cv2.applyColorMap(retYshow, 2) 
        # cv2.imwrite("demo_output/retY.jpg" , retYshow)  

  
    def fusionDepthCut(self, fusionXY,gradX , gradY):
        #需要在这里过滤原图像中的噪音，将竖条、横条全部去除，最后输出不同的分片
        #先扩展一下卷积看看效果？
        rows = fusionXY.shape[0]
        cols = fusionXY.shape[1]
        # expandConv=np.array([[1,1,1],[1,1,1],[1,1,1]])/9
        # expandConv=np.array([[1,1,1,1,1],[1,1,1,1,1],[1,1,1,1,1],[1,1,1,1,1],[1,1,1,1,1],])/16
        # # expandConv=np.array([[1,1],[1,1]])/4
        # # expandConv=np.array([[1,0,1],[1,0,1],[1,0,1]])
        # expandResult = signal.convolve2d(fusionXY , expandConv , 'same') 
        # expandResult = signal.convolve2d(expandResult , expandConv , 'same') #用来平滑融合图
        # expandResult = signal.convolve2d(expandResult , expandConv , 'same') 
        fusionXY = np.where(fusionXY>0 , 1. ,0. )
        kernel = np.ones((5,7), np.uint8)
 
        #图像闭运算
        expandResult = cv2.morphologyEx(fusionXY, cv2.MORPH_CLOSE, kernel)
        # expandResult = cv2.morphologyEx(expandResult, cv2.MORPH_CLOSE, kernel)
        # expandResult = cv2.morphologyEx(expandResult, cv2.MORPH_CLOSE, kernel)
        # expandResult = signal.convolve2d(expandResult , expandConv , 'same') 
        # expandResult = signal.convolve2d(expandResult , expandConv , 'same') 

        # expandResultX = gradX + expandResult
        # expandResultY = gradY + expandResult
        # #这段代码没有跑，可以留着做对比实验
        # expandResultShowX = np.where(expandResultX > (self.m_angleDivide_x*np.pi / 180 ) , 255 ,0 ).astype(np.uint8) #这里的5是调试出来的值
        # expandResultShowY = np.where(expandResultY > (self.m_angleDivide_y*np.pi / 180 ) , 255 ,0 ).astype(np.uint8) #这里的5是调试出来的值

        # # expandResultShow = ((expandResult-expandResult.min()) * 255 / (expandResult.max() - expandResult.min()) ).astype(np.uint8)
        # # expandResultShow = cv2.applyColorMap(expandResultShow, 2) 
        expandResult = np.where(expandResult > 0 , 255 , 0)
        cv2.imwrite("demo_output/expandResultShow.jpg" , expandResult)
        # cv2.imwrite("demo_output/expandResultShowX.jpg" , expandResultShowX) 
        # cv2.imwrite("demo_output/expandResultShowY.jpg" , expandResultShowY) 


        #这里是默认所有区域左侧的、上侧的占用较多
        tickColor = 1
        dividePic = np.zeros(expandResult.shape,np.uint64)
        for i in range(0 , rows) :
            for j in range(1 , cols) :
                if expandResult[i][j] == 0 and expandResult[i][j-1] == 0:
                    dividePic[i][j-1] = tickColor
                elif expandResult[i][j] == 0 and expandResult[i][j-1] > 0:
                    tickColor += 1 #区域变换，加色
                elif expandResult[i][j] >0 and expandResult[i][j-1] ==0 :
                    dividePic[i][j-1] = tickColor
            #换行直接换色
            tickColor += 1
        # print('divide X end,tickColor:' , tickColor , rows, cols)
        # expandResultShowX = np.where(dividePic > 0 , 0 , 255)

        #在这里乘以对应的mask，以避免边缘的融合？
        # dividePic = np.where(expandConv)
        # print(dividePic[4])
        #这里用0来表示边缘区域，计不计都行

        # devideShow = ((dividePic-dividePic.min()) * 255 / (dividePic.max() - dividePic.min()) ).astype(np.uint8)
        # devideShow = cv2.applyColorMap(devideShow, 2) 
        # devideShow = np.where(dividePic%2 == 1 , 255 , 0)
        # cv2.imwrite("demo_output/devideShowX.jpg" , devideShow)  

        # tickColor = 1
        # dividePic = np.zeros(expandResultShowX.shape,np.uint64)
        # for j in range(0 , cols) :
        #     click = 0
        #     for i in range(0 , rows) :
        #         if expandResultShowY[i][j] > 0  :
        #             click += 1
        #         else:
        #             if click > self.m_jumpPixelY :
        #                 tickColor+=1
        #             click = 0
        #         dividePic[i][j] = tickColor
        #     tickColor += 1
        
        # devideShow = np.where(dividePic%2 == 1 , 255 , 0)
        # cv2.imwrite("demo_output/devideShowY.jpg" , devideShow) 
        # exit()
        numPos = np.arange(0 , tickColor)
        minor = 0 
        # dividePic = np.where(expandResultShowY > 0 , 0 , dividePic) 
        for j in range(0, cols):
            for i in range(0 , rows-1):
                #仅对黑色的部分进行操作
                #先看分片编号是否一致，只有不一致的时候考虑是否需要融合
                if numPos[dividePic[i][j]] != numPos[dividePic[i+1][j]] :
                    if expandResult[i][j] == 0 and expandResult[i+1][j] == 0: #expandResult为0的地方才是平面
                    #判断累计变化角度是否大于阈值，若不大于则直接覆盖
                        minor = min( numPos[dividePic[i][j]] , numPos[dividePic[i+1][j]] )
                        numPos[dividePic[i][j]] = minor
                        numPos[dividePic[i+1][j]] = minor
        
        # print(dividePic)
                    # if expandResultShowY[i][j] > 0 :
                    #     click += 1
                    # else:
                    #     click = 0 
                    # if click <= self.m_angleDivide_y :#or expandResult[i+1][j] > 0:
                    #     minor = min( numPos[dividePic[i][j]] , numPos[dividePic[i+1][j]] )
                    #     numPos[dividePic[i][j]] = minor
                    #     numPos[dividePic[i+1][j]]  = minor
                    #判断角度差值是否大于需要分割的角度

        #分完片了，整合一下所有的块
        tick = 1
        numPosMap = np.zeros( tickColor)
        for k  in range(1 , tickColor):
            tmpNum = numPos[k]
            if numPosMap[tmpNum] == 0 and tmpNum != 0:
                numPosMap[tmpNum] = tick 
                tick += 1
        # print(numPosMap)
        # print(numPos,tickColor)
        numPosMap[numPos[dividePic[rows-2][cols-2]]] = 0
        numPosMap[numPos[dividePic[rows-2][1]]] = 0
        numPosMap[numPos[dividePic[1][cols-2]]] = 0
        numPosMap[numPos[dividePic[1][1]]] = 0
        for i in range(0 , rows) :
            for j in range(0 , cols) :
                dividePic[i][j] = numPosMap[numPos[dividePic[i][j]]]
        # print(dividePic)
        # print(tickColor,dividePic.max() , tick ,numPosMap.max())
        # multiple = 255.0 / tick
        # print(multiple)
        # print("after y-divide , tangle(180):" ,self.m_cut , ",tick num : " ,  tick , ",multiple : " , multiple )

        # return numPos , cutBlock , tick , multiple
        devideShow = ((dividePic-dividePic.min()) * 255 / (dividePic.max() - dividePic.min()) ).astype(np.uint8)
        devideShow = cv2.applyColorMap(devideShow, 2) 
        cv2.imwrite("demo_output/devideShow.jpg" , devideShow)  

        return dividePic , tick