import numpy as np
from CameraCut import CameraCutParam , CameraOuterParam
import cv2


class FlowHelper:
    def __init__(self , cols , rows ) :
        # xArray,yArray指示当前的位置在笛卡尔坐标系下的x轴,y轴坐标值，坐标系中心在影像坐标系的正中心
        self.cols = cols
        self.rows = rows
        self.xArray = np.arange(cols) 
        self.srcxArray = np.expand_dims(self.xArray , 0 ).repeat(rows , axis=0) 

        self.yArray = np.arange(rows).reshape(rows , 1) 
        self.srcyArray = np.repeat(self.yArray , cols , axis=1)+1e-5 #加上很小的角度，避免除0

        return
    
    def initCameraParam(self , camera1: CameraCutParam , camera2 : CameraCutParam , outCam : CameraOuterParam) :
        self.m_cam1 = camera1
        self.m_cam2 = camera2
        self.m_outCam = outCam 
        self.initOpticalFlowField()
        # print(self.pVec_unit)
        
        return
    
    def initOpticalFlowField(self) :
        #这里的值是针对源图像的固定值，后续不应进行修改
        self.xArray = self.srcxArray - self.m_cam1.m_cx #以像素数为单位，将图像坐标系转换至相机坐标系
        self.yArray = self.srcyArray - self.m_cam1.m_cy

        self.fi = np.arctan(self.xArray / self.yArray)
        self.r = np.sqrt(self.xArray**2 + self.yArray**2 + 1) #self.f **2) 
        
        self.sincita = 1/ self.r
        self.cita = np.arcsin(self.sincita) # 转换过程中默认认为z = 1 ，单位为像素 np.arcsin( self.f / self.r ) 
        self.cosbeta = np.cos(self.cita) * np.cos(self.fi)
        self.sinbeta = np.sin(np.arccos(self.cosbeta)) #这里数值上没什么问题
        zArray = np.ones((self.rows, self.cols , 1) ,float)  
        # zArray = zArray * self.m_cam1.m_fx
        xArray = self.xArray.reshape(self.rows , self.cols , 1)
        yArray = self.yArray.reshape(self.rows , self.cols , 1)
        
        self.P = np.concatenate((xArray , yArray , zArray ) , axis=2) #[[[x,y,z][x,y,z]]]  [    -601.06     -388.03      3252.3]
        self.pVec_unit = np.linalg.norm(self.P,axis=2)  #每个元素是(x,y,z)的长度坐标
        self.pVec_unit = self.P /  np.expand_dims(self.pVec_unit , axis= 2) #化为单位向量
        
        #关于外参的计算，计算两坐标轴之间的平移单位向量
        self.tVec_len = np.linalg.norm(self.m_outCam.m_t)
        self.tVec_unit = self.m_outCam.m_t / self.tVec_len


        # print(self.P)


    def PlaneCut(self,  flow_up , roiInfo = None) :
        return
        # exit()
        # nVec_unit = np.cross(vVec_unit , self.pVec_unit , axis=3) / self.sinbeta #pVec_unit：左图基准点到对应点的单位向量
        # print("nVec_unit:\n" , nVec_unit)
        # nVec_unit_transpose = np.transpose(nVec_unit)
        # print(nVec_unit_transpose)
        # point = np.dot(nVec_unit , nVec_unit_transpose)
        # print(point)
        

    def AngleCut(self , flow_up , roiInfo = None) :
        
        pass
        
    def Flow2Depth(self , flow_up , roiInfo = None) :
        #只有flow_up为规定的变量，其他值均为恒定值
        rows , cols = flow_up[0].shape
        zArray = np.ones((rows, cols , 1) ,float) 

        # xArray  = (flow_up[0]).reshape(rows , cols ,1 )
        # yArray = (flow_up[1] ).reshape(rows , cols ,1 )
        # vVec = np.concatenate((xArray, yArray , zArray), axis= 2) #估测的光流位置
        

        # depthMask = np.zeros((self.rows , self.cols ))
        if roiInfo ==None :
            xArray  = (flow_up[0] + self.srcxArray - self.m_cam2.m_cx).reshape(rows , cols ,1 )
            yArray = (flow_up[1] + self.srcyArray - self.m_cam2.m_cy).reshape(rows , cols ,1 )
            vVec = np.concatenate((xArray, yArray , zArray), axis= 2) #估测的光流位置
            vVec = vVec + self.P #做完计算的vVec代表右图中心点到对应点的距离 TODO
        else:
            # print("vVec not none")
            xArray  = (flow_up[0] + self.srcxArray[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])] - self.m_cam2.m_cx).reshape(rows , cols ,1 )
            yArray = (flow_up[1] + self.srcyArray[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])] - self.m_cam2.m_cy).reshape(rows , cols ,1 )
            vVec = np.concatenate((xArray, yArray , zArray), axis= 2) #估测的光流位置
            # vVec = vVec + self.P[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2]), :]
            # depthMask[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])] = 1
        #由原坐标系向目标坐标系的转变
        tmpvVec = np.swapaxes(vVec,0,2)
        w,h,c = vVec.shape
        tmpvVec=tmpvVec.reshape((c,-1))
        tmpvVec=self.m_outCam.m_R @ tmpvVec
        tmpvVec=tmpvVec.reshape((c,h,w))
        vVec = np.swapaxes(tmpvVec , 0 , 2)
        # self.m_outCam.m_R.multiple(tmpvVec,)

        # vVec =  vVec @ (self.m_outCam.m_R)  #转换到位移出来的的坐标系
        vVec_unit = np.linalg.norm(vVec,axis=2)
        vVec_unit = vVec / np.expand_dims(vVec_unit , axis=2) #左图中心到对应点的单位向量
        
        if roiInfo==None:
            cosbeta = np.sum(self.pVec_unit* vVec_unit,axis=2)
        else:
            cosbeta = np.sum(self.pVec_unit[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2]), :] * vVec_unit,axis=2) #降低维度至2维度，每个位置表示对应的cosbeta值
        beta = np.arccos(cosbeta)  #求出顶角大小
        sinbeta = np.sin(beta)


        cosalpha = np.sum(vVec_unit *(-self.m_outCam.m_t_unit) , axis=2)
        alpha = np.arccos(cosalpha )
        sinalpha = np.sin(alpha)

        depthplus = self.m_outCam.m_t_len * sinalpha /sinbeta
        if roiInfo == None :
            cutplus = depthplus /self.r #这就是对应的比例,也就是z方向上的深度信息
        else:
            cutplus = depthplus /self.r[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])] 
            #这里除以r主要是光心不一定垂直于图像中心
            
            # print(depthplus.shape ) #* np.sin(self.cita[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])])
            # print(depthplus)# np.sin(self.cita)) #这里的cita导致角度变化不太大 t:641mm
            # exit()
        # print(depthplus.max() , depthplus.min())
        # exit()
        # depthmax = 255

        # depthmask = np.where(depthplus > depthmax , 255 , 0) #边缘点会存在深度很高的点，这几句可以做可视化，也可以做统计
        # print("depthmask:" ,  depthmask.sum()/(cols*rows) * 100 /255) #去除的点在整个图像中的比例
        # cv2.imwrite("demo_output/depthmask.jpg" , depthmask)

        # depthplus = np.where(depthplus > depthmax , depthmax , depthplus) #边缘值非常大，尽可能去除

        #depthplus:实际的深度图，depthImg：放缩后为了可视化的深度图，colorImg：为了可视化，给可视化图上色的图
        # print(depthplus.max() , depthplus.min())
        # depthImg = ((depthplus- depthplus.min())*256 / (depthplus.max() - depthplus.min()) ).astype(np.uint8) 
        # colorImg = cv2.applyColorMap(depthImg, 2) 
        # cv2.imwrite("demo_output/colorImg.jpg" , colorImg) 
        return depthplus,cutplus

        #下面没调过
        # #先进行x,y坐标转换，找到flow_up中所有坐标在原图中的位置
        # rightXArray = flow_up[0] - self.m_cam1.m_cx #flow_up为当前坐标在左图中的动量参考，因此也用左图的内参 ，实际计算时两边的fx值相差不大，TODO
        # rightYArray = flow_up[1] - self.m_cam2.m_cy
        # left_unit = self.pVec_unit
        # left_cita = self.cita
        # left_r = self.r
        # if roiInfo != None : 
        #     rightXArray = roiInfo[0] + rightXArray
        #     rightYArray = roiInfo[1] + rightYArray
        #     left_unit = self.pVec_unit[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2]),:]
        #     left_cita = self.cita[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])]
        #     left_r = self.r[int(roiInfo[1]):int(roiInfo[3]) , int(roiInfo[0]):int(roiInfo[2])]
        
        # rightXArray = np.expand_dims(rightXArray , axis=2)
        # rightYArray = np.expand_dims(rightYArray , axis=2)
        # # print(flow_up.shape)  ([2, 536, 328]) = [channels , rows , cols]
        # rightZArray = np.ones((flow_up.shape[1] , flow_up.shape[2] ,1),float)
        # rightPoints = np.concatenate((rightXArray , rightYArray , rightZArray) , axis= 2)

        # #变为平面向量
        # right_unit = np.linalg.norm(rightPoints , axis=2)
        # right_unit = rightPoints / np.expand_dims(right_unit , axis=2 )
        # #下面开始计算两图中向量与t的夹角,根据夹角大小可根据正弦定理推测出两向量的长度，并计算深度
        # leftAngle = np.dot(left_unit , self.tVec_unit)
        # leftAngle = np.arccos(leftAngle) 

        # rightAngle = np.dot(right_unit , -self.tVec_unit)
        # rightAngle = np.arccos(rightAngle) 
        # topAngle = np.pi - leftAngle - rightAngle #顶角的大小
        # # leftLen / np.sin(leftAngle ) = self.tVec_len / np.sin(topAngle)
        # leftLen = np.sin(leftAngle)  / np.sin(topAngle) *self.tVec_len
        # picDepth = leftLen * np.sin(left_cita) * left_r #对应每个点的空间深度

        # return picDepth

        

        