import sys
sys.path.append('LeafHelper')

from typing import Any
import cv2
import numpy as np
import pandas as pd
import math
import random
import pathlib as Path
from CutHelper.CutHelper import CutType

class LeafParam:
    def __init__(self , leafOpex , leafRoot,  leafAngle) :
        self.m_leafOpex = leafOpex
        self.m_leafRoot = leafRoot
        self.m_leafAngle = leafAngle
        return


class LeafHelper:
    m_height = 0
    m_width = 0
    m_fx = 0.0
    m_fy = 0.0
    def __init__(self , sourceImg : cv2.Mat , fx: float , fy: float ) :
        size = sourceImg.shape
        self.m_height = size[0]
        self.m_width = size[1]
        self.m_fx = fx
        self.m_fy = fy
    
    def SetFloorAngle(self , floorAngle : float) : 
        self.m_fFloorCamAngle = floorAngle
    
    def DrawLeafIncs(self , leafParams : list , sourcePic: cv2.Mat):
        print('drawleafincs')
        for i , value  in enumerate(leafParams):
            color = [0,0,255]
            cv2.circle(sourcePic, value.m_leafRoot, 3, color, -1)
            cv2.line(sourcePic , value.m_leafRoot, value.m_leafOpex , color , 2)
            # cv2.putText(sourcePic , str(value.m_leafAngle),value.m_leafRoot , cv2.FONT_HERSHEY_SIMPLEX , 0.25 , color)
        cv2.imwrite("demo_output/angle.jpg" , sourcePic)
        return
    
    def setSavePath(self, inpath:str):
        self.m_saveP_ath = inpath

    def static(self , leafParams : list) :
        
        dataframe = pd.DataFrame([leafParams])
        dataframe.to_excel(self.m_saveP_ath)
        pass

# parser.add_argument('--depth_type' , type=int , default=3 , help='class for cutter:1:disparity;2:depth;3:optical flow')
    def CaculateAngle(self, cutterType:int , srcImg:cv2.Mat , cutSlices:list , depthPic , pluspic , roiInfo= None ):
        if roiInfo is not None :
            # print(roiInfo)
            inputImg =srcImg[int(roiInfo[1]):int(roiInfo[3]), int(roiInfo[0]):int(roiInfo[2]), 0]
        else:
            inputImg = srcImg[:,:,0]

        if cutterType == CutType.DepthCut.value :
            leafParam = self.caculateAngleWithDepthPic(inputImg , cutSlices ,depthPic , pluspic , roiInfo )
        
        if len(leafParam) > 0 :
            self.DrawLeafIncs(leafParam , srcImg)
        # self.static(leafParam , "")
        
    def caculateAngleWithDepthPic (self,srcImg:cv2.Mat, cutSlices: list , depthPic , cutplus ,roiInfo = None):
        retLeafs = []
        if len(cutSlices) <=0 :
            return retLeafs
        rows = cutSlices[0].shape[0]
        cols = cutSlices[0].shape[1]
        for k, img in enumerate(cutSlices) :
            dst = srcImg.copy()
            # print(dst.shape)
            dst[:,:] = img[:,:]
            skeleton = np.zeros(dst.shape, np.uint8)
            open_dst = np.zeros(dst.shape, np.uint8)
            while (True):
                if np.sum(dst) == 0:
                    break
                kernel = cv2.getStructuringElement(cv2.MORPH_CROSS, (7, 7))
                # dst = cv2.erode(dst, kernel, None, None, 1)
                cv2.erode(dst, kernel, dst , (-1,-1) , 1)
                cv2.morphologyEx(dst, cv2.MORPH_OPEN, kernel,open_dst )
                result = dst - open_dst
                skeleton = skeleton + result

            minPointX = cols - 1
            minPointY = rows-1
            maxPointX = 0 
            maxPointY = 0
            # cv2.imwrite("demo_output/skeleton" + str(k) + ".jpg" , skeleton)
            #寻找叶尖、叶根点
            for i in range(0,rows):
                for j in range(0,cols):
                    tColor = skeleton[i , j] 
                    if (tColor == 255 and (i < minPointY )):#or j < minPointX )) :
                        print("change",i,j)
                        minPointY = i
                        minPointX = j
                         
                    if( tColor == 255 and (i > maxPointY)):# or j > maxPointX)) :
                        maxPointY = i
                        maxPointX = j

            

            
            dA = depthPic[minPointY , minPointX] #叶根
            dB = depthPic[maxPointY , maxPointX] #叶尖
            plusA = cutplus[minPointY , minPointX]
            plusB = cutplus[maxPointY , maxPointX]

            distance = pow((minPointX - maxPointX) , 2 )+ pow((minPointY - maxPointY) , 2) #平面上的距离
            distance = math.sqrt(distance)
            pA = np.array([minPointX - self.m_width/2 , minPointY - self.m_height/2 , 1])
            pA = pA / np.linalg.norm(pA)
            pB = np.array([maxPointX - self.m_width/2 , maxPointY - self.m_height/2 , 1])
            pB = pB / np.linalg.norm(pB)
            # print(math.sqrt(469**2+223**2+1))

            # cosC = pA**2 +pB**2 - distance**2

            cosC = np.sum(pA * pB ) #/ np.linalg.norm(pA)*np.linalg.norm(pB)
            # print(pA * pB , cosC ,np.linalg.norm(cosC))
            

            lineA = distance * plusA
            lineB = dB * abs(1- plusA / plusB) 
            lineC = math.sqrt(dB**2 + dA **2 -2 * dB * dA * cosC)

            # angleA = math.acos((lineB**2 + lineC**2 - lineA**2) / (2*lineB*lineC))
            # print(lineC , lineB)
            if lineA == 0 or lineC==0 : 
                cosB = 0
            else:
                cosB =  (lineA**2 + lineC**2 - lineB**2) / (2*lineA*lineC)

            if cosB > 1 :
                cosB = 1 
            elif cosB < -1 :
                cosB = -1   #边界值的限制
            
            angleB = math.acos(cosB)
            
            # angleC = math.acos((lineA**2 + lineB**2 - lineC**2) / (2*lineA*lineB))

            # print(angleA , angleB , angleC , angleA + angleB+angleC)
                       
            # distanceB = pow((maxPointX - self.m_width/2),2) + pow((maxPointY-self.m_height/2),2)
            # distanceB = math.sqrt(distanceB)
            # symbol = np.sign((maxPointX-self.m_width/2) * (maxPointY-self.m_height/2))
            # tan = (dB-dA)/(distance*dA - (dB-dA)*distanceB*symbol)
            # tan = (dB - dA) / (distance*dA)
            # print(dB , dA , distance , tan)
            # print(math.atan(tan)*180/3.14)
            # angle = 90 - math.atan(tan) / 3.14 *180 - self.m_fFloorCamAngle
            if cosC < 0 :
                angle = 90 - self.m_fFloorCamAngle - (angleB/3.14 *180)
            else:
                angle = 90 - self.m_fFloorCamAngle + (angleB/3.14 *180)

            if angle > 90 :
                angle = 180 - angle
            # angle = ( (angleB) /3.14 *180) - self.m_fFloorCamAngle + 90
            
            #在原图中打标记
            if roiInfo != None :
                minPointX = minPointX + roiInfo[0]
                minPointY = minPointY + roiInfo[1]
                maxPointX = maxPointX + roiInfo[0]
                maxPointY = maxPointY + roiInfo[1]
                
            tmpLeaf = LeafParam((int(maxPointX),int(maxPointY)) , (int(minPointX),int(minPointY)) , angle)
            retLeafs+= [tmpLeaf]
            print("angle[",k,"] is : {%.4f}" %angle )


        return retLeafs

    #一系列经过二值化的slice表示叶片，而disparityPic为对应的角度
    def caculateAngleWithDisp (self, slices: list , dispartiyPic: np.array , roiInfo = None) :   
        retLeafs = []
        for i, img in enumerate(slices) :
            dst = img.copy()
            skeleton = np.zeros(dst.shape, np.uint8)
            open_dst = np.zeros(dst.shape, np.uint8)
            while (True):
                if np.sum(dst) == 0:
                    break
                kernel = cv2.getStructuringElement(cv2.MORPH_CROSS, (7, 7))
                # dst = cv2.erode(dst, kernel, None, None, 1)
                cv2.erode(dst, kernel, dst , (-1,-1) , 1)
                cv2.morphologyEx(dst, cv2.MORPH_OPEN, kernel,open_dst )
                result = dst - open_dst
                skeleton = skeleton + result

            minPointX = self.m_height - 1
            minPointY = self.m_width-1
            maxPointX = 0 
            maxPointY = 0

            #寻找叶尖、叶根点
            for i in range(0,self.m_height):
                for j in range(0,self.m_width):
                    tColor = skeleton[i , j] 
                    if (tColor == 255 and (i<minPointX or j <minPointY )) :
                        minPointX = i
                        minPointY = j
                         
                    if( tColor == 255 and (i > maxPointX or j > maxPointY)) :
                        maxPointX = i
                        maxPointY = j

            if roiInfo != None :
                minPointX = minPointX + roiInfo[0]
                minPointY = minPointY + roiInfo[1]
                maxPointX = maxPointX + roiInfo[0]
                maxPointY = maxPointY + roiInfo[1]

            
            dA = dispartiyPic[minPointX , minPointY]
            dB = dispartiyPic[maxPointX , maxPointY]
            distance = pow((minPointX - maxPointX) , 2 )+ pow((minPointY - maxPointY) , 2)
            distance = math.sqrt(distance)
        #     distance = pow((leafIncs[i].LeafOpex.x - leafIncs[i].LeafRoot.x),2) +  pow((leafIncs[i].LeafOpex.y - leafIncs[i].LeafRoot.y),2) ;
        # distance = sqrt(distance);

            # tan = self.m_fx*(dB-dA)/((dA-minPointX+self.m_width/2)*dB -(dB-maxPointX+93.*2)*dA)
            tan = (dA - dB) / distance
            angle = math.atan(tan) /3.14 *180 - self.m_fFloorCamAngle + 90
            tmpLeaf = LeafParam([maxPointX,maxPointY] , [minPointX, minPointY] , angle)
            retLeafs.append(tmpLeaf)
            print("angle[%d] is : %f " , i , angle)
        return retLeafs
        
