# LeafInclinationEstimate

#### Description
Auto-LIA is an efficient leaf inclination angle measurement system based on computer vision techniques and spatial geometric transformation. Specifically, first, the hardware part of the system consists of two relatively stationary RGB cameras with an inclinometer, and a set of input data consists of two RGB images with the angle between the hardware and the horizontal plane. Then, we combine object detection, as Yolov7, and 3D reconstruction such as RAFT for recovering leaf phenotypic information in the imaging space, which is the basis for subsequent calculations. Next, as for the accurate extraction of the key points of leaves, we propose a surface segmentation based on the gradient of spatial depth and angles, which implements the precision segmentation for leaves. Finally, we put forward a set of equations for the transformation from the spatial information of the leaves to LIA with the association between the image plane, the horizontal plane, and the target plant. The process we propose automates the entire process of non-invasive collection of leaf inclination angle, solves labor-intensive and invasive problems, and provides a strong correlation for individual plants.

#### Installation
Download the pre-trained model of yolov7 with RAFT shown on the page and place it in the current project directory.

#### Dependency

- Linux with Python >= 3.7
- matplotlib>=3.2.2
- numpy>=1.18.5,<=1.24.0
- opencv-python>=4.1.1
- Pillow>=7.1.2
- PyYAML>=5.3.1
- requests>=2.23.0
- scipy>=1.4.1
- torch>=1.7.0,!=1.12.0
- torchvision>=0.8.1,!=0.13.0
- tqdm>=4.41.0
- protobuf<=4.21.3
- pandas>=1.1.4



#### Instructions

# Hardware parameter settings:
Change the following contents of def main() in main.py to the hardware settings for image capturing.
```
srcCamera= CameraCutParam(2875.3,2868.62,715.189,350.285)
    rightCamera = CameraCutParam(3033.7,3020.2,707.717,328.642)
    R = np.array([[0.998876,0.00216193,0.0473521],
                [-0.00175768,0.999962,-0.00857723],
                [-0.0473688,0.00848436,0.998841]])
    t = np.array([-65.6515,-4.46654 ,-5.01777 ]) 
    outerParam = CameraOuterParam(R , t)
```

# Testing
```
python main.py --source ./demo_output/17_1_rectify.jpg --source2 ./demo_output/17_2_rectify.jpg --floor_angle 29.35 --filter_pixels_min 2000 --filter_pixels_max 50000 --detect_model_path yolov7.pth --depth_model_path raft_kitti.pth
```
