
import cv2
import argparse
# import glob
# import numpy as np
# import torch
# from tqdm import tqdm
# from pathlib import Path
# from PIL import Image
# from matplotlib import pyplot as plt
import numpy as np
import math
from DetectHelper.DetectHelper import DetectHelper 
from DepthHelper.DepthHelper import DepthHelper
from DepthHelper.core.utils import flow_viz
from CutHelper.CutHelper import CutHelper , CameraCutParam , CameraOuterParam
from LeafHelper.LeafHelper import LeafHelper
import time



DEVICE = 'cpu'

def detectObjEdit(srcXYXY , targetXYXY , width , height) :

    srcWidth = srcXYXY[2] - srcXYXY[0]
    srcHeight = srcXYXY[3] - srcXYXY[1]
    targetWidth = targetXYXY[2] - targetXYXY[0]
    targetHeight = targetXYXY[3] - targetXYXY[1]


    outputWidth = max(targetWidth , srcWidth)
    outputHeight = max(targetHeight , srcHeight)
    outputWidth = math.ceil(outputWidth / 8)
    outputHeight = math.ceil(outputHeight / 8)
    outputWidth = outputWidth * 8 
    outputHeight = outputHeight * 8

    srcPadWidth =  math.ceil((outputWidth - srcWidth) / 2)
    srcPadHeight = math.ceil((outputHeight - srcHeight) /2)
    leftPad = min(srcPadWidth , srcXYXY[0]) 
    if  (outputWidth - srcWidth - leftPad) > (width-srcXYXY[2]) : 
        rightPad = width - srcXYXY[2]
        leftPad = outputWidth - srcWidth - rightPad 
    else:
        rightPad = outputWidth - srcWidth - leftPad
    
    upPad = min(srcXYXY[1] , srcPadHeight)
    if (outputHeight - srcHeight - upPad) >  (height - srcXYXY[3]):
        downPad =  height - srcXYXY[3]
        upPad =  outputHeight - srcHeight - downPad
    else :
        downPad =  outputHeight - srcHeight - upPad
    srcXYXY[0] = srcXYXY[0] - leftPad
    srcXYXY[1] = srcXYXY[1] - upPad
    srcXYXY[2] = srcXYXY[2] + rightPad
    srcXYXY[3] = srcXYXY[3] + downPad

    targetPadWidth =  math.ceil((outputWidth - targetWidth) / 2)
    targetPadHeight = math.ceil((outputHeight - targetHeight) /2)
    leftPad = min(targetPadWidth , targetXYXY[0]) 
    if  outputWidth - targetWidth - leftPad > width-targetXYXY[2] : 
        rightPad = width - targetXYXY[2]
        leftPad = outputWidth - targetWidth - rightPad 
    else:
        rightPad = outputWidth - targetWidth - leftPad
    
    upPad = min(targetXYXY[1] , targetPadHeight)
    if outputHeight - targetHeight - upPad >  height - targetXYXY[3]:
        downPad =  height - targetXYXY[3]
        upPad =  outputHeight - targetHeight - downPad
    else :
        downPad =  outputHeight - targetHeight - upPad

    targetXYXY[0] = targetXYXY[0] - leftPad
    targetXYXY[1] = targetXYXY[1] - upPad
    targetXYXY[2] = targetXYXY[2] + rightPad
    targetXYXY[3] = targetXYXY[3] + downPad
        
    return srcXYXY , targetXYXY

DEVICE = 'cpu'
def main(args) :
    srcCamera= CameraCutParam(2875.3,2868.62,715.189,350.285)
    rightCamera = CameraCutParam(3033.7,3020.2,707.717,328.642)
    R = np.array([[0.998876,0.00216193,0.0473521],
                [-0.00175768,0.999962,-0.00857723],
                [-0.0473688,0.00848436,0.998841]])
    t = np.array([-65.6515,-4.46654 ,-5.01777 ]) 
    outerParam = CameraOuterParam(R , t)

    srcImg1 = cv2.imread(args.source)
    srcImg2 = cv2.imread(args.source2)
    imgSize = srcImg1.shape

    rows = imgSize[0] # height
    cols = imgSize[1] #width


    #load detect model
    detecter = DetectHelper(args.detect_thres , 640  ) 
    detecter.loadModule(args.detect_model_path , DEVICE)
    detecter.setClassName(args.detect_class)
    detecter.setRectColor(0,0,255)

    
    #load depth model
    depther = DepthHelper() 
    depther.loadModel(args.depth_model_path ,args , DEVICE)

    cutter = CutHelper(cols,rows ,srcCamera , rightCamera , outerParam , args.depth_type )
      
    cutter.SetAngleDivide(args.cut_angle_x , args.cut_angle_y) 
    cutter.SetJumpParam(args.jump_pixel_x , args.jump_pixel_y)
    cutter.SetPixelFilter(args.filter_pixels_min , args.filter_pixels_max)
    sourcePic = srcImg1

    leaves = LeafHelper(srcImg1 , srcCamera.m_fx , srcCamera.m_fy )
    leaves.SetFloorAngle(args.floor_angle)
    
    inputImg1 = srcImg1.copy()
    inputImg2 = srcImg2.copy()

    time_start_1 = time.time()

         
    img1 , img1XYXY = detecter.Detect(inputImg1 , args.detect_type)
    img2 , img2XYXY = detecter.Detect(inputImg2 , args.detect_type)

    if args.detect_type > 4 :
        img1XYXY , img2XYXY = None , None
        srcImg1 = img1
        srcImg2 = img2
    elif args.detect_type <= 2:    
        img1XYXY , img2XYXY = detectObjEdit(img1XYXY , img2XYXY , cols , rows )
        # srcImg1 = img1[]
        srcImg1 = img1[int(img1XYXY[1]):int(img1XYXY[3]), int(img1XYXY[0]):int(img1XYXY[2]), :] #裁剪时y范围在前，x范围在后
        srcImg2 = img2[int(img2XYXY[1]):int(img2XYXY[3]) , int(img2XYXY[0]):int(img2XYXY[2]), :]
        # print("rect",img1XYXY)
    else :
        srcImg1 = img1
        srcImg2 = img2
        img1XYXY , img2XYXY = None , None

    # cv2.imwrite("demo_output/detectImg1.jpg" , srcImg1)
    # cv2.imwrite("demo_output/detectImg2.jpg" , srcImg2)
    flow_up = depther.Estim(srcImg1 , srcImg2 , args.valid_iters ,1)
    retSlices , depthPic , pluspic =cutter.SegPic(flow_up[0] ,  args.cut_type , img1XYXY) #这里传入batch=1时的值
    leaves.CaculateAngle(2,sourcePic , retSlices , depthPic , pluspic , img1XYXY)

    time_start_2 = time.time()

    print("time cost:",(time_start_2-time_start_1),"s" )
    if args.show_pic :
        cv2.imshow("img1" , img1)
        cv2.imshow("img2" , img2)
        cv2.imshow("src" , srcImg1)
        cv2.imshow("target" , srcImg2)
        flo = flow_up[0].permute(1,2,0).cpu().numpy()
        flo = flow_viz.flow_to_image(flo)
        cv2.imshow("flow_up" , flo)
        cv2.waitKey(0)
   

    # cv2.waitKey(0) 

    return 


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    
    parser.add_argument('--mixed_precision', action='store_true', help='use mixed precision')
    parser.add_argument('--valid_iters', type=int, default=20, help='number of flow-field updates during forward pass')

    # Architecture choices
    parser.add_argument('--hidden_dims', nargs='+', type=int, default=[128]*3, help="hidden state and context dimensions")
    parser.add_argument('--corr_implementation', choices=["reg", "alt", "reg_cuda", "alt_cuda"], default="reg", help="correlation volume implementation")
    parser.add_argument('--shared_backbone', action='store_true', help="use a single backbone for the context and feature encoders")
    parser.add_argument('--corr_levels', type=int, default=4, help="number of levels in the correlation pyramid")
    parser.add_argument('--corr_radius', type=int, default=4, help="width of the correlation pyramid")
    parser.add_argument('--n_downsample', type=int, default=2, help="resolution of the disparity field (1/2^K)")
    parser.add_argument('--slow_fast_gru', action='store_true', help="iterate the low-res GRUs more frequently")
    parser.add_argument('--n_gru_layers', type=int, default=3, help="number of hidden GRU levels")
    
    parser.add_argument('--matrix_file_path', type=str, default="/data/jsj/dev/cpp/DisparityToCut/build-Debug/230423/chess1/checkMatrix.txt", help="file stores matrix params") #TODO
    parser.add_argument('--pics_path' , type=str , default="/data/jsj/dev/cpp/DisparityToCut/build-Debug/230423/plant1/rectify" , help='path of left and right imgs')
    parser.add_argument('--detect_model_path' , type = str , default='/data/jsj/dev/python/leaf-inclination-estimate/yolov7.pt' , help='path of yolov7')
    parser.add_argument('--depth_model_path', default='/data/jsj/dev/python/RAFT-master/models/raft-kitti.pth', help='location of depth estim model')
    parser.add_argument('--small' , default=False)


    parser.add_argument('--detect_type' , type=int , default=2 , help='operation after detect：1：GrayWithRect；2：ColorWithRect；3：GrayWithoutBackGround；4：ColorWithoutBackGround；5：DrawTemplate；6：DrawAllRects')
    parser.add_argument('--detect_thres' , type=float , default=0.0 , help='threshold for detect plants')
    parser.add_argument('--detect_class' , type=str , default='potted plant' , help='class for detect filter')
    parser.add_argument('--depth_type' , type=int , default=3 , help='class for cutter:1:disparity;2:depth;3:optical flow')

    parser.add_argument('--cut_type' , type=int , default=2 , help='class for cut strategy:1:LineGrad;2:ConvGrad;3:AngleGrad;4.PlaneGrad')
    parser.add_argument('--cut_angle_x' , type=float , default=.5, help='threshold for cut')
    parser.add_argument('--cut_angle_y' , type=float , default=.5, help='threshold for cut')
    parser.add_argument('--jump_pixel_x' , type=int , default=2 , help='pixels for jump at axis x')
    parser.add_argument('--jump_pixel_y' , type=int , default=2 , help='pixels for jump at axis y')
    parser.add_argument('--filter_pixels_min' , type=int , default=5000 , help='filter with pixels as pic size no less than this value')
    parser.add_argument('--filter_pixels_max' , type=int , default=10000 , help='filter with pixels as pic size no more than this value')
    parser.add_argument('--floor_angle' , type=float , default=25.6 )

    parser.add_argument('--source' , type=str , default='./demo_output/18_1_rectify.jpg')
    parser.add_argument('--source2' , type=str , default='./demo_output/18_2_rectify.jpg')

    parser.add_argument('--show_pic' , type=bool , default=False , help='if pics need to be seen' )

    args = parser.parse_args()

    main(args)